#ifndef _IOHANDLER_
#define _IOHANDLER_

#include <fstream>
#include <sstream>
#include <iomanip>

#include <vector>
#include <map>

using namespace std;

typedef struct  {
  string name;
  int pheno;
  double *covariates;
  bool *missing;          // flag - missing covariate needs to be estimated from Case/Ctrl mean
  double *evecs;
} Indiv;

class IOHandler  {

  public:

    // constructor
    IOHandler()  { beta = tbar = NULL; neigs = 0; }

    // destructor
    ~IOHandler()  { 

      // don't let IOHandler go out of scope while LTScore has pointers to beta and tbar 
      if ( beta != NULL )  { delete beta; }
      if ( tbar != NULL )  { delete tbar; }

      vector<Indiv *>::iterator p;
      for(p=iData.begin();p != iData.end();p++)  {
        Indiv *ip = *p;
        if ( ip->missing != NULL ) delete ip->missing;
        if ( ip->covariates != NULL ) delete ip->covariates;
        if ( ip->evecs != NULL ) delete ip->evecs;
        delete ip;
      }

    }


    // accessors
    int getParam(string name, string *val);
    vector<Indiv *> getIndivData(void)  { return iData; }
    int getNcovars(void);
    int getNeigs(void);
    void getCovarParams(double *mp, double **bp, double **tp)  {
      *mp = mval; *bp = beta; *tp = tbar;
    } 
    vector<string> getSnplist(void)  {  return snplist;  }

    // parse parameter file 
    int readParamFile(char *filename);

    // read EIGENSOFT style .ind file into 
    int readIndFile(void);

    // read EIGENSOFT style .evec file into 
    int readEvecFile(void);

    // read covariate parameter file
    int readCovariateParameters(void);

    // find the indices of the genetic covariates
    int *findSNPs(const char *filename);

    // DEBUGGING
    void dbgShowParams(void);
    void dbgShowIndivs(void);
    void dbgShowCovarParams(void);


  private:

    void setParam(string key, string val);
    map<string,string> rpar;
    vector<Indiv *> iData;
    vector<string> snplist;

    int neigs;

    // covariate parameters
    double *beta;
    double *tbar;
    double mval;

};




#endif

