#ifndef _LTFIT_
#define _LTFIT_

#include <iostream>
#include <cstdlib>
#include <cmath>

#include "LTFitIOHandler.h"

class LTFit  {

  public:
    LTFit(LTFitIOHandler *h)  { 
      ioh = h;
      thresholdisset = false; 
      outisopen = false;
      thoutput = false;
    };

    ~LTFit(void)  {
      if ( outisopen )  {
        outstream.close();
      }
    }

    // LTPub algorithm 
    int fitClinicalCovariateData();
    void writeClinicalCovariateData(const char *filename);
    int fitGeneticCovariateData(void);
    void setThresholdFlag()  {  threshold = -threshold; thresholdisset = true;  }

    ofstream outstream;
    bool outisopen;


  private:

    LTFitIOHandler *ioh;   
    
    int ncovar;                  // number of clinical covariates
    Covariate **covariates;      // clinical covariate data

    bool thresholdisset;
    double threshold;

    int nindiv;
    int *phenotypes;
    double fexp, fcase, fctrl;
    double prevalence;

    int inverseDnorm(double y, double *result);
    double dnorm(double x);
    void leastSquaresSolution(void);
    double evalF(void);
    double evalF(double *);

    int readGenotype(int *g, int n, ifstream *gin);
    double minimize(int *ierr);
    int inverse(double y, double *result);
    double pnorm(double x, double mu, double sigma, int lower_tail);
    double f(double x);

    bool thoutput;     // kludge


};

#endif
