#ifndef _LTFITIOHANDLER_
#define _LTFITIOHANDLER_

#include <fstream>
#include <sstream>
#include <iomanip>

#include <vector>
#include <map>

#include "Covariate.h"

using namespace std;

class LTFitIOHandler  {

  public:

    // constructor
    LTFitIOHandler()  {  
      ncovar = 0;
      covariates = NULL;
      nindiv = 0;
      phenotypes = NULL;
    }

    // destructor
    ~LTFitIOHandler()  {  
       // need to free covariates (list of lists)
       // also pheno
     }

    // accessors
    int getParam(string name, string *val);
    vector<string> getSnplist(void)  { return snplist; }    // genetic covariates, by index in .geno file

    int getNcovars(void)  {  return ncovar;  }
    Covariate **getCovariates()  {  return covariates;  }  // list of covariate data from data file
 
    int getNindiv(void)  {  return nindiv;  }
    int *getPhenotypes(void)  {  return phenotypes;  }

    // parse parameter file 
    int readParamFile(const char *filename);

    // read SNP file to find indices of genetic covariates
    int *findSNPs(const char *filename);

    // read clinical covariate data file
    int readDataFile(const char *filename);

    // read .ind file 
    int readIndFile(void);

    // debugging
    void dbgShowParams(void);
    void dbgShowCovariateData(void);
    void dbgShowIndivs(void);


  private:

    void setParam(string key, string val);
    map<string,string> rpar;
    vector<string> snplist;       // genetic covariates, by index in .geno file

    Covariate **covariates;    // array of pointers to type Covariate from reading data file
    int ncovar;

    int *phenotypes;
    int nindiv;

};




#endif

