#ifndef _LTPUB_
#define _LTPUB_

#include <cmath>
#include <cstdlib>
#include <vector>
#include <iostream>
#include <iomanip>
#include <fstream>

#include "StringTokenizer.h"
#include "LinearSolve.h"

using namespace std;

#define DEBUG 0

class DataPoint  {
  public:
    DataPoint(int n)    { T = new double[n]; missing = new int[n]; } 
    double *T;          // array of size nindices for the covariate this belongs to (t_ijk)
    int *missing;       // array of size
    double sigma;       // sigma_ij
};

class Covariate  {
  public:
    Covariate(int n)  { ndatapts = 0; nindices = n; tbar = new double[n]; C = new double[n]; data = NULL; }
    int ndatapts;      // N_i
    int nindices;      // M_i
    double *tbar;      // array of size nindices (tbar_ij)
    double *C;         // array of size nindices (C_ij)
    DataPoint **data;  // array of data points of size ndatapts 
};

class LTPub  {

  public:
    LTPub(void)  { ncovar = 0; covariates = NULL; }
    // remember to write ~LTPub(void);
    int readInput(char *filename);
    int run(void);
    void writeOutput(char *filename);
    void dbgGridSearch(void);

  private:
    int ncovar;
    Covariate **covariates;
    double mval;

    int inverseDnorm(double y, double *result); 
    double dnorm(double x);
    void leastSquaresSolution(void);
    double evalF(void);

    void exhaustiveGridSearch(double *fminp, double *cval, double *cmin, double *minsrch, double *maxsrch, double *delsrch, int n);
    void dbgShowInput(void);
    double evalF(double *);
    int ncval;      // for exhaustive search

};

#endif
