#ifndef _LTSCORE_
#define _LTSCORE_

#include <iostream>
#include <fstream>
#include <sstream>
#include <cstdlib>

#include "IOHandler.h"

class LTScore  {

  public:

    LTScore(void);
    ~LTScore(void);

    int unpackData(IOHandler *ioh);
    int run(IOHandler *ioh);

  private:
    int estimateMissingCovariates(void);
    int computeLiability(double *x);
    int readGenotype(int *g, int n, ifstream *gin);
    int adjustOLSvecs(void);
    void meanAdjustGeno();
    void adjustOLSGeno();
    int correlation(double *X, double *Y, double *result);    
    int loadGeneticCovariates(IOHandler *ioh);

    inline int computeATTScore(int *G, int *P, int n, double *result);
    inline int computeLTScore(int *G, int *P, double *X, int n, double *result);
    inline int computeEIGScore(int *G, int *P, int n, double *result);
    inline int computeEIGLTScore(int *G, int *P, double *x, int n, double *result);
    inline int computeOLSScore(int *G, int *P, int n, double *result);
    inline int computeOLSLTScore(int *G, int *P, double *x, int n, double *result);

    int ncovar;    
    int ngcovar;    
    int neigs;    
    int nsamples;    

    int *pheno;          // integer-valued phenotype - also used to detect missing data 
    int *geno;           // integer-valued genotype - also used to detect missing data

    double *gamma;       // mean-adjusted genotype
    double *gammaadj;    // genotype adjusted with covariates for OLS
    double *phi;         // mean-adjusted phenotype
    double *phiadj;      // phenotype adjusted with covariates for OLS
    double *xi;          // mean-adjusted posterior means
    double *xiadj;       // posterior means adjusted with covariates for OLS

    int **gcovariates;  // genetic covariates
    int *gcovarindices; // indices in .geno and .snp file of genetic covariates
    double **covariates;   // clinical covariates
    bool **missing;        
    double **evecs;
    double *xtx;

    double mval;
    double *beta;     // effect-sizes
    double *tbar;     // means 

    int idbg;  // DEBUG

};

#endif

// Note: how do we handle "NA" in an eigenvalue? Set it to zero?
// Note: make correlation subroutine, instread of replicating code 

// bad data flags:
//  0 : skip if ( P[i] <= -1 || G[i] == 9    (ATT,LT)
//  1 : skip if ( P[i] <= -1 || G[i] == 9 || evecs[i] == NULL )(EIG,EIGLT,OLS,OLS-LT) 
