#include <iostream>
#include <cstdlib>

#include "LTFitIOHandler.h"
#include "LTFit.h"

using namespace std;

int main(int argc, char **argv)  {

  if ( argc != 2 )  {
    cout << "usage:  ltscore <parameter-file>" << endl;
    return 1;
  }

  LTFitIOHandler ioh;
  LTFit ltf(&ioh);

  // read parameter file
  if ( ioh.readParamFile(argv[1]) )  {
    cerr << "aborted" << endl;
    exit(1);
  }
  // ioh.dbgShowParams();

  // if the problem has clinical covariates, need an output file for the parameters
  string ccf;     // covariate parameter output file
  string idf;     // individual input file
  string snf;     // SNP input file
  string dtf;     // clinical covariate input file

  vector<string> snplist = ioh.getSnplist();               // list of genetic covariates

  int ic = (!ioh.getParam("datafile",&dtf));                // clinical covariates
  int ig = ( snplist.size() > 0 );

  if ( ic || ig )  {
    int ih = ioh.getParam("outfile",&ccf);
    if ( ih )  {   // no output file, use default filename 
      cerr << "warning: no output file name for covariate data in parameter file\n";
      cerr << "covariate parameters will be written to covar.param.out\n";
      ccf = "covar.param.out";
    }
  }
  
  // if the problem has genetic covariates, need a SNP file and an indiv file
  if (  ig )  {
    int ih = ioh.getParam("indivfile",&idf);
    if ( ih )  {  
      cerr << "error: genetic covariates require individual file name in parameter file\n";
      exit(1);
    }
    ih = ioh.getParam("snpfile",&snf);
    if ( ih )  {  
      cerr << "error: genetic covariates require SNP file name in parameter file\n";
      exit(1);
    }
  }

  if ( ic )  {                                             // if there are clinical covariates
    ioh.readDataFile(dtf.c_str());                         // read data about clinical covariates
    // ioh.dbgShowCovariateData();                            // print it out so we can check it
    if ( ltf.fitClinicalCovariateData() == 1 )  {          // use least-squares to find effect sizes
      cerr << "error: can't find clinical covariate effect sizes\n";
      exit(1);
    }
    ltf.writeClinicalCovariateData(ccf.c_str());
    ltf.setThresholdFlag();                                // threshold was set in LS step
  }

  if ( snplist.size() > 0 )  {
    ioh.readIndFile();
    if ( ltf.fitGeneticCovariateData() == 1 )  {
      cerr << "error: can't fit genetic covariates" << endl;
    }
  }

}


