#include "LinearSolve.h"

#include <iostream>
#include <iomanip>

LinearSolve::LinearSolve(double *A, double *b, double *x, int n)  {


  // unpack
  gsl_matrix *amat = gsl_matrix_alloc(n,n);
  gsl_vector *bvec = gsl_vector_alloc(n);
  gsl_vector *xvec = gsl_vector_alloc(n);

  for(int i=0;i<n;i++)  {
    gsl_vector_set(xvec, i, x[i]);
    gsl_vector_set(bvec, i, b[i]);
    for(int j=0;j<n;j++)  {
      gsl_matrix_set(amat, i, j, A[i*n+j]);
    }
  }

  // solve 
  gsl_permutation *p = gsl_permutation_alloc(n);
  int sgn;

  gsl_linalg_LU_decomp(amat, p, &sgn);
  gsl_linalg_LU_solve(amat, p, bvec, xvec);

  // repack
  for(int i=0;i<n;i++)  {
    x[i] = gsl_vector_get(xvec,i);    
  }

  gsl_vector_free(bvec);
  gsl_vector_free(xvec);
  gsl_matrix_free(amat);
  gsl_permutation_free(p);

}

